/* 
 * HomeSwitch v1.0.0 made by Ewald Kendziorra sr http://kendziorra.nl 
 * Copyright 2013-2014. All rights reserved.
 * License: Free
 * Included files: client.ino, diversen.ino, instellingenEEPROM.ino, kakuVoorArduino.ino, 
 *                 lcd.ino, schakelem.ino, sensoren.ino
 * extra libraries: LiquidCrystal_I2C, DS1307RTC, Time, RemoteTransmitter, RemoteReceiver,
 *                  TimeDate, Temp_Lux_Sensors, SDSPI
 *                            ===============================
 *                            USE HomeSwitch AT YOUR OWN RISK
 *                            ===============================
 ************************************************************************************
 *****                                HomeSwitch                                *****
 ***** Licht, screen(s), zonwering, gordijnen en andere voorzieningen schakelen *****
 ***** Verlichting en gordijnen schakelen bij zonOP/zonOnder (samen veiligheid) *****
 ***** Screen(s) afhankelijk van temperatuur in woonkamer en zon besturen       ***** 
 *****           rolluik(en) kan natuurlijk ook                                 *****
 *****           voor uitval-zonwering benodigd een windsnelheidsmeting  (N/A)  *****
 ************************************************************************************/

/***** OPTIES voorwaardelijk compileren *****/
//**** met een UNO kunnen zijn niet alle logische combinaties mogenlijk!! (ERROR: Sketch too big...)
//**** SDCARD en CLIENT samen is niet logisch (zelfde functie)
#define SCREEN  // Screen(s) zonwering met RF-AB of KaKu: voorwaardelijk compileren
//#define KAKU  // SCREEN + KAKU: ALLEEN voor zonwering met KaKu zonweringontvanger (dus zonder RF-AB)
#define LCD  // 4x20 LCD I2C display 
//#define GORDIJN  // Gordijn(en) bij zonOP openen en sluiten bij zonOnder
//#define RGBLED  // Optionele RGB-LED voor KaKu-mode Auto/Aan/Uit/Stop 
#define CLIENT  // voor instellingen met Windows of Android App
//#define SDCARD  // voor instellingen vanaf SD 

/***** libraries *****/
#include <EEPROM.h>
#include <Wire.h>
#include <DS1307RTC.h>
#include <Streaming.h>
#include <TimeDate.h>  // TimeDate library? on my site: http://www.kendziorra.nl/arduino
#include <Time.h>
#include <RemoteTransmitter.h>
#include <RemoteReceiver.h>
#ifdef LCD
  #include <LiquidCrystal_I2C.h>
#endif
#ifdef SDCARD   // SDSPI library? on my site: http://www.kendziorra.nl/arduino
  #include <SPI.h>
  #include <Unsigned.h>
  #include <sdraw.h>
  #include <sdcard.h>
  #include <ff.h>
#endif
#ifdef SCREEN
  #include <MCP9701temp.h>  // Temp_Lux_Sensors library? on my site: http://www.kendziorra.nl/arduino
  #include <GL5528lux.h>
#endif

/*### SET YOUR Location (lib: TimeDate), Time zone and DST/Summer time offsets ###
 *================================================================================
 * the Centre of the Netherlands, timezone UTC/GMT+1, lat 52.0, lon 5.0
 * EU Summer Time (DST) (explanation in TimeDate library readme.h): 
 *   start at the last sunday of Mar. on 02:00, offset +1h (add dstOffset to local time)
 *   ends at the last sunday of Oct. on  03:00, offset -1h (subtract dstOffset from local time) */
const float latitude = 52.0, longitude = 5.0;  // in deximals, used for Surise and Sunset
const byte dstStartMonth =  3, dstStartMonthDay = 0, dstStartWeek = 0, dstStartWeekDay = 1, dstStartHour = 2,  // DST start date rules
           dstEndMonth   = 10, dstEndMonthDay   = 0,   dstEndWeek = 0,   dstEndWeekDay = 1,   dstEndHour = 3;  // DST end date rules
const long utcOffset = 3600;  // UTC/GMT offset in seconds (EU = UTC+1)
const int dstOffset = 3600;  // DST/Summer Time offset in seconds (EU = EU time zone +1)

/***** compiler contstants *****/
const byte KNOP_1 = 0, KNOP_2 = 1, KNOP_3 = 2, KNOP_4 = 3, KNOP_5 = 4, KNOP_6 = 5, KNOP_7 = 6, KNOP_8 = 7;  // KaKuVoorArduino knoppen
const byte SCHAKEL = 0, DIMMER = 1;  // voor ontvangers[][]
const byte TEMP_1 = 0, LUX_1 = 1, TEMP_2 = 2, LUX_2 = 3;
const byte tempPin = A0;  // sensorpin temp
const byte luxPin = A1;  // sensorpin lux
const byte kakuPin = 11;  // sensorpin lux
const byte led13Pin = 13;  // brand als lampen aan zijn
const byte SpiSsPin = 10;  // SDCard SS

/*================================================================*/

/******************************************************************************
 ******** START instellen voor de te schakelen KaKu-ontvangers         ********
 ******** WIJZIGEN WERKT ALLEEN NA UPLOAD VAN SKETCH "resetEEPROM.ino" ********
 ********    of gebruik de Windows of Android App voor het wijzigen    ********/
//### in dit voorbeeld: Kaku huiscode C, ontvanger (knop) #1, is geen dimmer, alleen avonds en om 03:00 uit
char huisCode = 'C';  // huiscode KaKu (type char tussen ' NIET " 
/* te schakelen ontvangers
 * [SCHAKEL][n] = 0/1 niet/wel schakelen ([schakel/dimmer][n] "n" is een paar)
 * [DIMMER][n] = 0/1 niet/wel isDIMMER  
 * knop 
 *1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 */
byte ontvangers[2][16]= {
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
byte ochtend = 0, avond = 1;  // licht schakelen? 0=NEE, 1=JA
byte aanUur = 7, aanMinuut = 0;  // ochtends: licht AAN om
byte uitUur = 3, uitMinuut = 0;  // avonds:   licht UIT om
int aanOmMinuten = 0, uitOmMinuten = 0, sunRiseMinuten = 0, sunSetMinuten = 0;  // hulpjes
byte zonOffset = 30;  // minuten - licht eerder UIT/AAN (ochtend/avond)
/******** EINDE instellen voor de te schakelen KaKu-ontvangers ********
 **********************************************************************/

unsigned long utcTime, localTime, localTimeOffset;
boolean lampenZijnAan = false; 
volatile boolean knopFlag[8];  // flag voor RF-ontvangst van KaKu-AB, knoppen 1 t/m 8
/* waarden voor knopfuncties 1 t/m 8  
 * 0=false 1=true of 0-1-2-3 huidige opdrachtcode (Auto/Aan/Uit/Stop) */
byte knopWaarde[8];  
byte saveMinuut = 60;  // eens per minuut test; in loop()
int nuMinuten;
String sInput;  // USB-Serial input buffer

#ifdef SCREEN  // SCREEN-waarden met "// ***** ...... *****" zonodig hier aanpassen
#ifndef KAKU  // optocouplers of KaKu
const byte optoPinNeer = 6;  // 2 optocouplers voor screen RF-AB
const byte optoPinOp = 7;
#else
const char huisCodeScreen = 'D';  // ***** KaKu huiscode voor Screen *****
const byte kakuKnopScreen = 1;  // ***** KaKu knop voor screen *****
#endif
const float tempOffset = 0.0;  // ***** compensatie temp (proefondervindelijk vaststellen) *****
const byte schakelTemp = 21;  // ***** temp schakelpunt screen Neer ***** (screen Op bij schakelTemp-1.0)
const byte luxOffset = 0;  // ***** compensatie Lux (proefondervindelijk vaststellen) ***** 
const int schakelLuxNeer = 350;  // ***** Lux schakelpunt voor screen Neer (proefondervindelijk vaststellen) *****
const int schakelLuxOp = 160;  // ***** Lux schakelpunt voor screen Op (proefondervindelijk vaststellen) *****
boolean screenIsNeer = false, screenTemp = false, screenLux = false;  // hulpjes
byte telSens_1 = 0, telSens_2 = 0;  // voor gemiddelde metingtingen sensoren
// voor gemiddelde metingtingen sensoren
float saveSens[4][10]; 
float temp = 0, lux = 0;  // temp en lux (gem. over 10x loop() metingen)
#endif
#ifdef GORDIJN
const char huisCodeGordijn = 'D';  // ***** KaKu huiscode voor Gordijn *****
const byte kakuKnopGordijn = 2;  // ***** KaKu knop voor Gordijn *****
const byte kakuKnopZonwering = 3;  // ***** KaKu knop voor Zonwering avonds ophalen *****
#endif
#ifdef LCD
/***** LCD-tekens ***** zon, maan, lamp, screen, temp, lux, graad */
const uint8_t tekensLCD[7][8] = {
  0x04, 0x15, 0x0E, 0x1F, 0x0E, 0x15, 0x04, 0x00,
  0x0C, 0x18, 0x18, 0x18, 0x18, 0x0C, 0x07, 0x00,
  0x04, 0x04, 0x0E, 0x1F, 0x1F, 0x1F, 0x0E, 0x00,
  0x1F, 0x1F, 0x1F, 0x1F, 0x11, 0x11, 0x1F, 0x00,
  0x0E, 0x11, 0x11, 0x15, 0x15, 0x15, 0x0E, 0x00,
  0x10, 0x10, 0x10, 0x10, 0x15, 0x12, 0x15, 0x00,
  0x0C, 0x12, 0x12, 0x0C, 0x00, 0x00, 0x00, 0x00};  
LiquidCrystal_I2C lcd(0x27, 20, 4);  // set LCD op I2C-adres 0x27 en 20x4 display
#endif
#ifdef RGBLED  // rgbLED kleuren
const int R = 5, G = 9, B = 10;  // anode poorten
const int rgb[4][3] = { 
  5, 0, 0,
  0, 2, 0,
  0, 0, 5,
  7, 0, 12}; 
#endif

static inline int dagMintuten(int h, int m) {return h * 60 + m;}  // aantal aan minuten in hh:mm

KaKuTransmitter kaKuTransmitter(kakuPin);  // Instance KaKuSwitch AB,  pin D11

void setup()
{
  Serial.begin(9600);  //38400);  //115200);
  delay(2000);
  pinMode(led13Pin, OUTPUT);  // Arduino LED-13 brand bij lampen aan en gordijnen dicht
  char h = EEPROM.read(0);
  char s = EEPROM.read(1);
  if (h != 'H' || s != 'S')
  {  // test of instellingen in EEPROM staan
    EEPROM.write(0, 'H');  // HS = HomeSwitch 
    EEPROM.write(1, 'S'); 
    schrijfEEPROM();  // save instellingen in EEPROM (die welke bovenaan staan)
  } 
  else leesEEPROM();  // laad instellingen uit EEPROM
#ifdef SCREEN 
  Temp.begin(tempPin);
  Temp.setTempComp(tempOffset);
  Lux.begin(luxPin);
  Lux.setLuxComp(luxOffset);
#ifndef KAKU
  pinMode(optoPinNeer, OUTPUT);
  pinMode(optoPinOp, OUTPUT);
  screenOpto(optoPinOp);
#endif  
#ifdef RGBLED
  pinMode(R, OUTPUT); // poorten
  pinMode(G, OUTPUT);
  pinMode(B, OUTPUT);
#endif  
#endif  
  RemoteReceiver::init(0, 3, kakuVoorArduino);  // interrupt 0 (pin D2), 3 repeats, callback function
  /***** KLOK GELIJKZETTEN ***** (kan ook met de Windows of Android App) *****/
//  setTime(0, 0, 0, 1, 1, 14);  // UTC!!!
//  RTC.set(now());  // RTC en system tijd instellen
  setSyncProvider(RTC.get);  // the function to get the time from the RTC
  setSyncInterval(60);  // set the number of seconds between re-sync
  if(timeStatus() != timeSet) beeps(20);  // RTC loopt niet!
#ifdef LCD  
  lcd.init();  // init lcd 
  doKnop_8();  // lcd backlight aan
  // eigen LCD-tekens
  lcd.createChar(0, (uint8_t*)tekensLCD[0]);  // zonOP zon
  lcd.createChar(1, (uint8_t*)tekensLCD[1]);  // zonOnder maan
  lcd.createChar(2, (uint8_t*)tekensLCD[2]);  // KaKu lamp
  lcd.createChar(3, (uint8_t*)tekensLCD[3]);  // Screen rechthoek
  lcd.createChar(4, (uint8_t*)tekensLCD[4]);  // LDR thermometer
  lcd.createChar(5, (uint8_t*)tekensLCD[5]);  // NTC lx
  lcd.createChar(6, (uint8_t*)tekensLCD[6]);  // graden °
#endif  
  TD.initDst(dstStartMonth, dstStartMonthDay, dstStartWeek, dstStartWeekDay, dstStartHour,  // DST start date rules
  dstEndMonth,   dstEndMonthDay,   dstEndWeek,   dstEndWeekDay,   dstEndHour,  // DST end date rules
  utcOffset, dstOffset);  // time zone and DST in seconds
  TD.initSun(latitude, longitude, utcOffset);  // init Sun with location in decimals and time zone in seconds
  getTime();
  initKaKu(true);
}

void loop()
{
  getTime();
  if (saveMinuut != minute(utcTime))
  {  // 1x per minuut uitvoeren is voldoende
#ifdef SDCARD
    testSD();  // kijk of instellingen moeten worden gewijzigd
#endif
    saveMinuut = minute(utcTime);
    if (hour(localTime) == 4 && minute(localTime) == 0)
    {  // nieuwe dag om 04:00 ;-)
      if (knopWaarde[KNOP_1] != 3) 
      {  // om 04:00, behalve bij Stop,
        knopWaarde[KNOP_1] = 0;  // licht op Auto
#ifdef RGBLED
        rgbLed();
#endif
        lampenUit();  // en lampen uit
#ifdef GORDIJN
        gordijnDicht();  // gordijnen dicht
#endif
      }
#ifdef SCREEN  
      if (knopWaarde[KNOP_2] != 3)
      {  // om 04:00, behalve bij Stop,
        knopWaarde[KNOP_2] = 0;  // screen op Auto
#ifndef KAKU  // screen Op
        screenOpto(optoPinOp);
#else
        screenKaKu(false);
#endif
      }
#endif  
    }
    // testen of lampen aan of uit moeten zijn en gordijn dicht of open
    // morgends
    if (nuMinuten == aanOmMinuten && ochtend == 1 && knopWaarde[KNOP_1] == 0) lampenAan();
    else if (nuMinuten == sunRiseMinuten)
    {
      if (ochtend == 1 && knopWaarde[KNOP_1] == 0) lampenUit();
#ifdef GORDIJN
      gordijnOpen();  // zonOp: ook als het licht niet wordt geschakeld
#endif
    }
    // avonds
    if (nuMinuten == sunSetMinuten)
    {
      if (avond == 1 && knopWaarde[KNOP_1] == 0) lampenAan();
#ifdef GORDIJN
      gordijnDicht();  // zonOnder: ook als het licht niet wordt geschakeld
#endif
    }
    else if (nuMinuten == uitOmMinuten && avond == 1 && knopWaarde[KNOP_1] == 0) lampenUit();
#ifdef LCD  
    toonDatumTijd();  // datum en tijd naar LCD
    toonLampMode();  // KaKu is in mode: Auto of Aan of Uit of Stop
    toonZonMaan();  // zon op en ondergang
#endif  
#ifdef SCREEN
    meetSens(true);  // temp en luxx meten en verwerken
    screenNeerOP();
#ifdef LCD
    toonOpNeer();
    toonScreenMode();  // Screen is in mode: Auto of Neer of Op of Stop
#endif  
#endif  
  }
  // voorkom met een Flag het onodig uitvoeren van KaKu=>Arduino knopfuncties
  if (knopFlag[KNOP_1]) doKnop_1();
#ifdef SCREEN  
  if (knopFlag[KNOP_2]) doKnop_2();
  meetSens(false);  // temp en lux meten maar niet verwerken
#endif  
#ifdef LCD  
  if (knopFlag[KNOP_8]) doKnop_8();
  lcd.setCursor(16, 0);  // positie ":"
  if (second(localTime) % 2) lcd << ' ';  // (second MODULO 2 geeft rest van second/2) 1=" " 0=":" (per sec. aan/uit)
  else lcd << ':';
#endif  
  // Serial.println(freeRam());  // test staat met '//' in diversen.ino
  delay(700);
}

void getTime()
{
  utcTime = now();  // zet datum en tijd vast in "utcTime" ***** de klok staat altijd op UTC
  localTimeOffset = TD.localTimeOffset(utcTime);  // time zone offset (and DST offset?), isDst, startDst and endDst are also calculated 
  TD.CalcSunTimes(day(utcTime), month(utcTime), year(utcTime), TD.isDst ? dstOffset : 0);  // calculate SunRise and SunSet in local time
  localTime = utcTime + localTimeOffset; 
  nuMinuten = dagMintuten(hour(localTime), minute(localTime));
  sunRiseMinuten = dagMintuten(TD.sunRiseHour, TD.sunRiseMinute + zonOffset);
  sunSetMinuten = dagMintuten(TD.sunSetHour, TD.sunSetMinute - zonOffset);
  aanOmMinuten = dagMintuten(aanUur, aanMinuut);
  uitOmMinuten = dagMintuten(uitUur, uitMinuut);
}

/***** bij start of na reset: dagwaarden berekenen en testen of er geschakeld moet worden *****/
void initKaKu(boolean reset)
{
  if (ochtend)
  {  // en of lampen aan of uit moeten zijn
    if (nuMinuten >= aanOmMinuten && nuMinuten < sunRiseMinuten) lampenAan();
    else lampenUit(); 
  }
  if (avond)
  {
    /* let op: de "nieuwe dag" begint hier pas om 04:00uur (zie ook in loop())
     * tijden tussen 24:00 en 04:00 krijgen dus een offset van 24 uur */
    if (uitUur < 4) uitOmMinuten += dagMintuten(24, 0);  // na 24:00?
    if (hour(localTime) < 4) nuMinuten += dagMintuten(24, 0);  // na 24:00?
    if (nuMinuten >= sunSetMinuten && nuMinuten < uitOmMinuten) lampenAan();
    else lampenUit(); 
  }
#ifdef SCREEN  
  if (reset)
  {
    // alleen na reset, init temp en lux met 1e sensorwaarden
    temp = Temp.getTemp();
    lux = Lux.getLux();
    for (byte i = 0; i < 10; i++)
    {
      saveSens[TEMP_1][i] = temp; 
      saveSens[TEMP_2][i] = temp; 
      saveSens[LUX_1][i] = lux; 
      saveSens[LUX_2][i] = lux; 
    }
  }
#endif  
#ifdef RGBLED
  rgbLed();
#endif
  saveMinuut = 60;  // loop() een onbekende minuut voorspiegelen (bekent = 0-59)
}

