/********************************************************
 ***** Arduino/HomeSwitch met een KaKu-AB bedienen  *****
 ********************************************************/

void kakuVoorArduino(unsigned long receivedCode, unsigned int period)
{
  /* KaKu voor Arduino interrupt, uitlezen van knopcode en de bijbehorende knopFlag_n zetten
   * in een interrupt kun je geen fucties aanroepen, dus in functie "loop" knopFlag_n lezen */
  /* knoppen 1 t/m 8: Aan1-Uit2, Aan3-Uit4, Aan5-Uit6, Aan7-Uit8
   * 1) licht: Auto/Aan/Uit/Stop (1/2/3/4 beepjes) bij Aan of Uit om 03:00 weer Uit en Auto
   * 2) screen: Auto/Neer/Op/Stop (1/2/3/4 beepjes) bij Neer of OP om 03:00 weer Op en Auto
   * 8) LCD-backlight: Aan/Uit
   * globale variabelen hier gewijzigd: als volatile declareren
   * -------                                --------
   * knopFlag[knop1-7] wordt in de functie loop() getest */
  switch (receivedCode)
  {  // huiscode: E  groep: III 
  case 39554:  // 1
    knopFlag[KNOP_1] = true;  // lampen
    break;
  case 39552:  // 2
    knopFlag[KNOP_2] = true;  // screen
    break;
    //  case 43928:  // 3
    //  knopFlag[KNOP_3] = true;
    //  break;
    //  case 43926:  // 4
    //  knopFlag[KNOP_4] = true;
    //  break;
    //  case 41012:  // 5
    //  knopFlag[KNOP_5] = true;
    //  break;
    //  case 41010:  // 6
    //  knopFlag[KNOP_6] = true;
    //  break;
    //  case 45386:  // 7
    //  knopFlag[KNOP_7] = true;
    //  break;
#ifdef LCD  
  case 45384:  // 8
    knopFlag[KNOP_8] = true;  // LCD-backlight
#endif  
  }
}

void doKnop_1()
{
  /* verlichting Auto/Aan/Uit/Stop
   * indien mode Aan of Uit dan om 03:00 weer op Auto */
  knopFlag[KNOP_1] = false;
  knopWaarde[KNOP_1]++;
  if (knopWaarde[KNOP_1] == 4)
    knopWaarde[KNOP_1] = 0;
  beeps(knopWaarde[KNOP_1] + 1);
#ifdef RGBLED
  rgbLed();
#endif
  switch (knopWaarde[KNOP_1])
  {
  case 0: 
    initKaKu(false);  // licht "Auto"
    break;
  case 1: 
    lampenAan();  // licht handmatig "Aan" 
    break;
  case 2:
    lampenUit();  // licht handmatig "Uit"
    break;
  }
#ifdef LCD  
  toonLampMode();
#endif
}

#ifdef RGBLED  // moet in een functie want het wijzigd op meerdere plaatsen 
void rgbLed()
{  // kleur LED aanpassen
  analogWrite(R, rgb[knopWaarde[KNOP_1]][0]);  // r waarde uit rgb[n] 
  analogWrite(G, rgb[knopWaarde[KNOP_1]][1]);  // g waarde uit rgb[n] 
  analogWrite(B, rgb[knopWaarde[KNOP_1]][2]);  // b waarde uit rgb[n] 
}
#endif

#ifdef SCREEN
void doKnop_2() 
{
  /* screen Auto/Neer/Op/Stop
   * indien mode Neer of Op dan om 03:00 weer op Auto */
  knopFlag[KNOP_2] = false;
  knopWaarde[KNOP_2]++;
  if  (knopWaarde[KNOP_2] == 4)
    knopWaarde[KNOP_2] = 0;
  beeps(knopWaarde[KNOP_2] + 1);
  switch (knopWaarde[KNOP_2])
  {
  case 1: 
#ifndef KAKU  
    screenOpto(optoPinNeer);  // screen handmatig "Neer"
#else
    screenKaKu(true);
#endif
    screenIsNeer = true;
    break;
  case 2:
#ifndef KAKU  
    screenOpto(optoPinOp);  // screen handmatig "Op"
#else
    screenKaKu(false);
#endif
    screenIsNeer = false;
  }
#ifdef LCD  
  toonScreenMode();
  toonOpNeer();
#endif
}
#endif

#ifdef LCD
void doKnop_8()
{
  // backlight Aan/Uit
  knopFlag[KNOP_8] = false;
  knopWaarde[KNOP_8] = !knopWaarde[KNOP_8];  // invert knop4
  lcd.setBacklight(knopWaarde[KNOP_8]);
}
#endif



