#include <SPI.h>
#include <ff.h>
#include <sdcard.h>
#include <Unsigned.h>

FATFS FatFs;  // Work area (file system object) for logical drive
SdFatFs sdFatFs;  // SD FatFs class
SdSpi sdSpi;  // SD SS pin

void showError(FRESULT fr);
FRESULT writeFile (char* fName, char* fTxt, uInt bytes);
FRESULT readFile (char* fName);

void setup()
{
  Serial.begin(9600);
  sdSpi.pinSS((uChar)10);  // SD Select pin
  delay(2000);  // wait for Serial Monitor 8n1
  FRESULT fr;  // FatFs return code

  fr = sdFatFs.f_mount(&FatFs, "", 0);  // init SD or test if memory card is inserted
  if (fr) showError(fr);
  else
  {
    Serial.println(F("\r\n#### Read text from READ.TXT ####"));
    fr = readFile((char*)"read.txt");  // read a text file
    if (fr) showError(fr);
    Serial.println(F("\r\n#### Write/Add text to WRITE.TXT ####"));
    char n[] = "write.txt", w[] = "This text is added to the file.\r\n";
    fr = writeFile(n, w, sizeof(w) - 1);  // write to text file - strip null character ('\0') from w
    if (fr) showError(fr); 
    Serial.println(F("\r\n#### Read added text back ####"));
    fr = readFile(n);  // read the text back
    if (fr) showError(fr);
  }
}

void loop()
{
  //  delay(50);
}

FRESULT readFile (char* fName)
{
  FIL fil;  // File object
  char line[128];  // Line buffer
  FRESULT fr;  // FatFs return code

  fr = sdFatFs.f_open(&fil, fName, FA_READ);  // open file
  if (fr) return fr;
  while (sdFatFs.f_gets(line, sizeof(line), &fil))  // read en print file
    Serial.print(line);
  Serial.println();
  sdFatFs.f_close(&fil);  // Close file
  return FR_OK;
}

FRESULT writeFile (char* fName, char* fTxt, uInt bytes)
{
  FIL fil;  // File object
  uInt b;  // bytes written
  FRESULT fr;  // FatFs return code

  //fr = sdFatFs.f_open(&fil, fName, FA_CREATE_ALWAYS);  // Creates a new file. If the file is existing, it will be truncated and overwritten.
  fr = sdFatFs.f_open(&fil, fName, FA_OPEN_ALWAYS);  // Opens the file if it is existing. If not, a new file is created.
  if (fr) return fr;
  fr = sdFatFs.f_lseek(&fil, f_size(&fil));  // Move read/write pointer to end of file to append text
  if (fr) return fr;
  fr = sdFatFs.f_write(&fil, fTxt, bytes, &b);   // append text to file
  sdFatFs.f_close(&fil);  // Close file
  return fr;
}

void showError(FRESULT fr)
{
  Serial.print(F("\t ERROR: ")); Serial.println(fr);  // enum FRESULT error code in ff.h
}
