#ifndef FF_h
#define FF_h

#include <Unsigned.h>

/*---------------------------------------------------------------------------/
 / Functions and Buffer Configurations
 /---------------------------------------------------------------------------*/

#define  _USE_STRFUNC  1  /* 0:Disable or 1-2:Enable */
/* To enable string functions, set _USE_STRFUNC to 1 or 2 (strip \r). */

/*---------------------------------------------------------------------------/
 / Drive/Volume Configurations
 /---------------------------------------------------------------------------*/

#define _FS_NOFSINFO  0  /* 0 to 3 */
/* If you need to know correct free space on the FAT32 volume, set bit 0 of this
 /  option and f_getfree() function at first time after volume mount will force
 /  a full FAT scan. Bit 1 controls the last allocated cluster number as bit 0.
 /
 /  bit0=0: Use free cluster count in the FSINFO if available.
 /  bit0=1: Do not trust free cluster count in the FSINFO.
 /  bit1=0: Use last allocated cluster number in the FSINFO if available.
 /  bit1=1: Do not trust last allocated cluster number in the FSINFO.
 */

/*---------------------------------------------------------------------------/
 / System Configurations
 /---------------------------------------------------------------------------*/

#define  _FS_LOCK  2  /* 0:Disable or >=1:Enable */
/* To enable file lock control feature, set _FS_LOCK to 1 or greater.
 /  The value defines how many files/sub-directories can be opened simultaneously.
 /  This feature consumes _FS_LOCK * 12 bytes of bss area. */

/* Definitions of volume management */

#define LD2PD(vol) (uChar)(vol)  /* Each logical drive is bound to the same physical drive number */
#define LD2PT(vol) 0  /* Find first valid partition or in SFD */

typedef char TCHAR;

/* File system object structure (FATFS) */

struct FATFS{
  uChar  fs_type;  /* FAT sub-type (0:Not mounted) */
  uChar  drv;  /* Physical drive number */
  uChar  csize;  /* Sectors per cluster (1,2,4...128) */
  uChar  n_fats;  /* Number of FAT copies (1 or 2) */
  uChar  wflag;  /* win[] flag (b0:dirty) */
  uChar  fsi_flag;  /* FSINFO flags (b7:disabled, b0:dirty) */
  uShort id;  /* File system mount ID */
  uShort n_rootdir;  /* Number of root directory entries (FAT12/16) */
  uLong  last_clust;  /* Last allocated cluster */
  uLong  free_clust;  /* Number of free clusters */
  uLong  n_fatent;  /* Number of FAT entries (= number of clusters + 2) */
  uLong  fsize;  /* Sectors per FAT */
  uLong  volbase;  /* Volume start sector */
  uLong  fatbase;  /* FAT start sector */
  uLong  dirbase;  /* Root directory start sector (FAT32:Cluster#) */
  uLong  database;  /* Data start sector */
  uLong  winsect;  /* Current sector appearing in the win[] */
  uChar  win[512];  /* Disk access window for Directory, FAT (and file data at tiny cfg) */
};

/* File object structure (FIL) */

struct FIL{
  FATFS*  fs;  /* Pointer to the related file system object (**do not change order**) */
  uShort  id;  /* Owner file system mount ID (**do not change order**) */
  uChar  flag;  /* File status flags */
  uChar  err;  /* Abort flag (error code) */
  uLong  fptr;  /* File read/write pointer (Zeroed on file open) */
  uLong  fsize;  /* File size */
  uLong  sclust;  /* File data start cluster (0:no data cluster, always 0 when fsize is 0) */
  uLong  clust;  /* Current cluster of fpter */
  uLong  dsect;  /* Current data sector of fpter */
  uLong  dir_sect;  /* Sector containing the directory entry */
  uChar*  dir_ptr;  /* Pointer to the directory entry in the window */
#if _FS_LOCK
  uInt  lockid;  /* File lock ID (index of file semaphore table Files[]) */
#endif
};

/* Directory object structure (DIR) */

struct DIR{
  FATFS*  fs;  /* Pointer to the owner file system object (**do not change order**) */
  uShort  id;  /* Owner file system mount ID (**do not change order**) */
  uShort  index;  /* Current read/write index number */
  uLong  sclust;  /* Table start cluster (0:Root dir) */
  uLong  clust;  /* Current cluster */
  uLong  sect;  /* Current sector */
  uChar*  dir;  /* Pointer to the current SFN entry in the win[] */
  uChar*  fn;  /* Pointer to the SFN (in/out) {file[8],ext[3],status[1]} */
#if _FS_LOCK
  uInt  lockid;  /* File lock ID (index of file semaphore table Files[]) */
#endif
};

/* File status structure (FILINFO) */

struct FILINFO{
  uLong  fsize;  /* File size */
  uShort  fdate;  /* Last modified date */
  uShort  ftime;  /* Last modified time */
  uChar  fattrib;  /* Attribute */
  TCHAR  fname[13];  /* Short file name (8.3 format) */
};

/* File function return code (FRESULT) */

enum FRESULT{
  FR_OK = 0,  /* (0) Succeeded */
  FR_DISK_ERR,  /* (1) A hard error occurred in the low level disk I/O layer */
  FR_INT_ERR,  /* (2) Assertion failed */
  FR_NOT_READY,  /* (3) The physical drive cannot work */
  FR_NO_FILE,  /* (4) Could not find the file */
  FR_NO_PATH,  /* (5) Could not find the path */
  FR_INVALID_NAME,  /* (6) The path name format is invalid */
  FR_DENIED,  /* (7) Access denied due to prohibited access or directory full */
  FR_EXIST,  /* (8) Access denied due to prohibited access */
  FR_INVALID_OBJECT,  /* (9) The file/directory object is invalid */
  FR_WRITE_PROTECTED,  /* (10) The physical drive is write protected */
  FR_INVALID_DRIVE,  /* (11) The logical drive number is invalid */
  FR_NOT_ENABLED,  /* (12) The volume has no work area */
  FR_NO_FILESYSTEM,  /* (13) There is no valid FAT volume */
  FR_MKFS_ABORTED,  /* (14) The f_mkfs() aborted due to any parameter error */
  FR_TIMEOUT,  /* (15) Could not get a grant to access the volume within defined period */
  FR_LOCKED,  /* (16) The operation is rejected according to the file sharing policy */
  FR_NOT_ENOUGH_CORE,  /* (17) LFN working buffer could not be allocated */
  FR_TOO_MANY_OPEN_FILES,  /* (18) Number of open files > _FS_SHARE */
  FR_INVALID_PARAMETER  /* (19) Given parameter is invalid */
};

#define f_eof(fp) (((fp)->fptr == (fp)->fsize) ? 1 : 0)
#define f_error(fp) ((fp)->err)
#define f_tell(fp) ((fp)->fptr)
#define f_size(fp) ((fp)->fsize)

#define EOF (-1)

/*--------------------------------------------------------------*/
/* Additional user defined functions  */

/* RTC function */
uLong get_fattime (void);

/*--------------------------------------------------------------*/
/* Flags and offset address  */

/* File access control and file status flags (FIL.flag) */

#define  FA_READ  0x01
#define  FA_OPEN_EXISTING  0x00
#define  FA_WRITE  0x02
#define  FA_CREATE_NEW  0x04
#define  FA_CREATE_ALWAYS  0x08
#define  FA_OPEN_ALWAYS  0x10
#define FA__WRITTEN  0x20
#define FA__DIRTY  0x40

/* FAT sub type (FATFS.fs_type) */

#define FS_FAT12  1
#define FS_FAT16  2
#define FS_FAT32  3

/* File attribute bits for directory entry */

#define  AM_RDO  0x01  /* Read only */
#define  AM_HID  0x02  /* Hidden */
#define  AM_SYS  0x04  /* System */
#define  AM_VOL  0x08  /* Volume label */
#define AM_LFN  0x0F  /* LFN entry */
#define AM_DIR  0x10  /* Directory */
#define AM_ARC  0x20  /* Archive */
#define AM_MASK  0x3F  /* Mask of defined bits */

/* Fast seek feature */
#define CREATE_LINKMAP  0xFFFFFFFF

/*--------------------------------*/
/* Multi-byte word access macros  */

#define  LD_uShort(ptr)  (uShort)(((uShort)*((uChar*)(ptr)+1)<<8)|(uShort)*(uChar*)(ptr))
#define  LD_uLong(ptr)  (uLong)(((uLong)*((uChar*)(ptr)+3)<<24)|((uLong)*((uChar*)(ptr)+2)<<16)|((uShort)*((uChar*)(ptr)+1)<<8)|*(uChar*)(ptr))
#define  ST_uShort(ptr,val)  *(uChar*)(ptr)=(uChar)(val); *((uChar*)(ptr)+1)=(uChar)((uShort)(val)>>8)
#define  ST_uLong(ptr,val)  *(uChar*)(ptr)=(uChar)(val); *((uChar*)(ptr)+1)=(uChar)((uShort)(val)>>8); *((uChar*)(ptr)+2)=(uChar)((uLong)(val)>>16); *((uChar*)(ptr)+3)=(uChar)((uLong)(val)>>24)

class SdFatFs
{
  public:
  FRESULT f_open (FIL* fp, const TCHAR* path, uChar mode);  /* Open or create a file */
  FRESULT f_close (FIL* fp);  /* Close an open file object */
  FRESULT f_read (FIL* fp, void* buff, uInt btr, uInt* br);  /* Read data from a file */
  FRESULT f_write (FIL* fp, const void* buff, uInt btw, uInt* bw);  /* Write data to a file */
  FRESULT f_lseek (FIL* fp, uLong ofs);  /* Move file pointer of a file object */
  FRESULT f_sync (FIL* fp);  /* Flush cached data of a writing file */
  FRESULT f_mount (FATFS* fs, const TCHAR* path, uChar opt);  /* Mount/Unmount a logical drive */
  int f_putc (TCHAR c, FIL* fp);  /* Put a string to the file */									/* Put a character to the file */
  int f_puts (const TCHAR* str, FIL* cp);  /* Put a string to the file */
  int f_printf (FIL* fp, const TCHAR* str, ...);  /* Put a formatted string to the file */
  TCHAR* f_gets (TCHAR* buff, int len, FIL* fp);  /* Get a string from the file */
};

#endif




