// Explanation FatFs funtions see below
// ------------------------------------

// SdSpi.ino
// =========
/*-------------------------------------------------------------------------------------/
 /  (C)ChaN, FatFs for Arduino, Spark Core and compatible 
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 /-------------------------------------------------------------------------------------/
 / Serial Monitor 8n1 for Arduino, Spark Core and comaptible
 / ==================
 / download: www.kendziorra.nl/arduino
 /-------------------------------------------------------------------------------------/
 /  Installed CODE PAGE is 1252, Latin 1 (Windows) (in ff.cpp)
 /      other code pages in file ff.c in download from elm-chan.org/fsw/ff/ff10a.zip
 /------------------------------------------------------------------------------------*/

// sdcard
// ======
/*--------------------------------------------------------------------------------------------/
 /  Low level SD card routines used by FatFs  (C)ChaN, 2013
 /--------------------------------------------------------------------------------------------/
 /  Modified and ported to software SPI for Arduino, Spark Core and compatible 
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 / 
 /  Modify header files, SPI port and SS pin for Arduino, Spark Core or other microcontroller  
 /--------------------------------------------------------------------------------------------
 / Serial Monitor 8n1 for Arduino, Spark Core and comaptible
 / ==================
 / download: www.kendziorra.nl/arduino
 /-------------------------------------------------------------------------------------------*/

//SdFlash
//sdraw
//=======
/*-------------------------------------------------------------------------------------/
 /  SD as Flash memory for Arduino, Spark Core and compatible 
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 /-------------------------------------------------------------------------------------/
 / Serial Monitor 8n1 for Arduino, Spark Core and comaptible
 / ==================
 / download: www.kendziorra.nl/arduino
 /-------------------------------------------------------------------------------------/
 / **** <sdcard.cp> and <sdcard.h> from my <SdSpi.ino> FatFs SD file system ****
 /-------------------------------------------------------------------------------------/
 / * You can use any SD or SDHC card <= 32GB. I use a 2GB FAT (= FAT16) formatted
 / SD card and started read/write at first data sector 512. The card will
 / remain usable for normal use, your raw data is then overwritten.
 / * For SD cards > 2GB and/or other formats as FAT16 the first data sector is NOT 512
 /-------------------------------------------------------------------------------------/
 / * Raw and files mixed:
 / Format SD and create as first one file. For example myData.bin with 64KB of dummy
 / data. Then and you have, only on FAT16, reserved 2 cluster that's 128 sectors of
 / 512 bytes. Then you can use sectors 512 to 640 voor raw and the other SD space for
 / other files. 
 / * Raw and files mixed on other formats as FAT16:
 / You need to find out where the first data sector begins, how many bytes a sector
 / contains and how many sectors per cluster.
 / Quite useful for this: Active@ Disk Editor - Advanced Disk Editor(Hex Viewer)
 /   FREEWARE lsoft.net/disk-editor.aspx (use the themplates)
 /-------------------------------------------------------------------------------------*/

// ff
// ==
/*--------------------------------------------------------------------------------------------/
 /  FatFs - FAT file system module include file  R0.10a  (C)ChaN, 2014
 /--------------------------------------------------------------------------------------------/
 /  Modified for Arduino, Spark Core and compatible 
 /  by Ewald Kendziorra sr, 2014, www.kendziorra.nl
 /--------------------------------------------------------------------------------------------/
 / Serial Monitor 8n1 for Arduino, Spark Core and comaptible
 / ==================
 / download: http://www.kendziorra.nl/arduino
 /--------------------------------------------------------------------------------------------/
 / FatFs module is a generic FAT file system module for small embedded systems.
 / This is a free software that opened for education, research and commercial
 / developments under license policy of following terms.
 /
 /  Copyright (C) 2014, ChaN, all right reserved.
 /
 / * The FatFs module is a free software and there is NO WARRANTY.
 / * No restriction on use. You can use, modify and redistribute it for
 /  personal, non-profit or commercial product UNDER YOUR RESPONSIBILITY.
 / * Redistributions of source code must retain the above copyright notice.
 /
 /--------------------------------------------------------------------------------------------/
 / ##################################################################
 / ####### many ChaN FatFs universal definitions are stripped #######
 / ##################################################################
 /
 / ################################################
 / ####### put your own code page in ff.cpp #######
 / ################################################
 / Installed CODE PAGE is 1252, Latin 1 (Windows) (in ff.cpp)
 /      other code pages in file ff.c http://elm-chan.org/fsw/ff/ff10a.zip
 / The _CODE_PAGE specifies the OEM code page to be used on the target system.
 / Incorrect setting of the code page can cause a file open failure.
 /--------------------------------------------------------------------------------------------/
 / * SdFat FatFs module provides following functions to the applications:
 /   ====================================================================
 / f_mount - Register/Unregister a work area            elm-chan.org/fsw/ff/en/mount.html
 / f_open - Open/Create a file-                         elm-chan.org/fsw/ff/en/open.html
 /          FA_READ, FA_WRITE, FA_OPEN_EXISTING, FA_CREATE_NEW, FA_CREATE_ALWAYS
 / f_close - Close an open file                         elm-chan.org/fsw/ff/en/close.html
 / f_read - Read file                                   elm-chan.org/fsw/ff/en/read.html
 / f_write - Write file                                 elm-chan.org/fsw/ff/en/write.html
 / f_lseek - Move read/write pointer, Expand file size  elm-chan.org/fsw/ff/en/lseek.html
 / f_sync - Flush cached data                           elm-chan.org/fsw/ff/en/sync.html
 / f_gets - Read a string                               elm-chan.org/fsw/ff/en/gets.html
 / f_putc - Write a character                           elm-chan.org/fsw/ff/en/putc.html
 / f_puts - Write a string                              elm-chan.org/fsw/ff/en/puts.html
 / f_tell - Get current read/write pointer              elm-chan.org/fsw/ff/en/tell.html
 / f_eof - Test for end-of-file on a file               elm-chan.org/fsw/ff/en/eof.html
 / f_size - Get size of a file                          elm-chan.org/fsw/ff/en/size.html
 / f_error - Test for an error on a file                elm-chan.org/fsw/ff/en/error.html
 / f_printf - Write a formatted string                  elm-chan.org/fsw/ff/en/printf.html
 /         f_printf(&fil, "%d", 1234);             "1234"
 /         f_printf(&fil, "%6d,%3d%%", -200, 5);   "  -200,  5%"
 /         f_printf(&fil, "%ld", 12345L);          "12345"
 /         f_printf(&fil, "%06d", 25);             "000025"
 /         f_printf(&fil, "%06d", -25);            "000-25"
 /         f_printf(&fil, "%-6d", 25);             "25    "
 /         f_printf(&fil, "%u", -1);               "65535" or "4294967295"
 /         f_printf(&fil, "%04x", 0xAB3);          "0ab3"
 /         f_printf(&fil, "%08LX", 0x123ABCL);     "00123ABC"
 /         f_printf(&fil, "%016b", 0x550F);        "0101010100001111"
 /         f_printf(&fil, "%s", "String");         "String"
 /         f_printf(&fil, "%8s", "abc");           "     abc"
 /         f_printf(&fil, "%-8s", "abc");          "abc     "
 /         f_printf(&fil, "%c", 'a');              "a"
 /         f_printf(&fil, "%f", 10.0);             f_printf lacks floating point support
 /--------------------------------------------------------------------------------------------/
 / * Append to file? Use f_lseek and f_size
 /  char fileName[] = "write.txt", txtToWrite[] = "This text is added to the file";
 /  // Opens the file if it is existing.If not, a new file is created
 /  fr = f_open(&fil, fileName, FA_OPEN_ALWAYS);
 /  fr = f_lseek(&fil, f_size(&fil));  // Move read/write pointer to end of file
 /  // append to file, strip null character ('\0') 
 /  fr = f_write(&fil, txtToWrite, sizeoff(txtToWrite) - 1, &b);
 /--------------------------------------------------------------------------------------------*/
