#include "sdraw.h"

SdSpi __sdSpi;  // SD I/O class

/*-------------------------------------------
 init SD or test if memory card is inserted
 --------------------------------------------*/
DSTATUS SdRaw::initSd()
{
  return __sdSpi.disk_initialize();  // init SD or test if memory card is inserted
}

/*--------------------------------
  fill entire sector with a char
 ---------------------------------*/
DRESULT SdRaw::clearRaw (uLong sector, char chr)
{
  uChar b[sectorSize];
  for (uInt i = 0; i < sectorSize; i++) b[i] = chr;
  return __sdSpi.disk_write (b, sector, (uInt)1);  // write one sector
}

/*-------------------------------------
 write string or String(number) to SD
 --------------------------------------*/
DRESULT SdRaw::writeString(String str, uLong sector, uInt offset)
{
  char bw[str.length() + 1];
  str.toCharArray(bw, str.length() + 1);
  return writeRaw((uChar*)bw, sector, offset, str.length());
}

/*--------------------------------
 write a raw block of data to SD
 ---------------------------------*/
DRESULT SdRaw::writeRaw (const uChar* buff, uLong sector, uInt offsetInSector, uInt bytesToWrite)
{
  DRESULT fr;  // return code
  uChar b[sectorSize];

  fr = __sdSpi.disk_read (b, sector, (uInt)1);  // read one sector
  if (fr) return fr;
  for (uInt i = 0; i < bytesToWrite; i++) b[offsetInSector + i] = buff[i];  // set bytes
  return __sdSpi.disk_write (b, sector, (uInt)1);  // write one sector
}

/*------------------------------
 read a raw block data from SD
 -------------------------------*/
DRESULT SdRaw::readRaw (uChar* buff, uLong sector, uInt offsetInSector, uInt bytesToRead)
{
  DRESULT fr;  // return code
  uChar b[sectorSize];

  fr = __sdSpi.disk_read(b, sector, (uInt)1);  // read one sector
  if (fr) return fr;
  for (int i = 0; i < bytesToRead; i++) buff[i] = b[offsetInSector + i];  // get bytes
  return RES_OK;
}
