#ifndef SDRAW_h
#define SDRAW_h

#if defined(ARDUINO)
  #if ARDUINO >= 100
	  #include "Arduino.h"
  #else
	  #include "WProgram.h"
  #endif
#elif defined(SPARK_CORE)
	#include "application.h"
#endif

#include <Unsigned.h>
#include "sdcard.h"

// ********************************************************************************
// sector and cluster data for FAT (=FAT16) SD <= 2GB
// Raw and files mixed, data file MYRAW.TXT of 64KB (2 clusters = 128 sectors) 
const uLong firstSector = 512;  // first data sector on SD FAT16 (SD <= 2GB)  
const uLong lastSector = 640;  // last data sector on SD FAT16 (SD <= 2GB) is 64KB 
const uInt sectorSize = 512;  // number of bytes per sector (standard for SD)
const uInt clusterSize = 64; // number of sectors per cluster
// ********************************************************************************

// ********************************************************************************
// ATTENTION: one sector = 512 bytes, offsetInSector + bytesToRead <= 512 !!!
// ********************************************************************************
class SdRaw
{
  public:
  DSTATUS initSd();  // init SD or test if memory card is inserted
  DRESULT clearRaw (uLong sector, char chr);  // fill entire sector with chr
  DRESULT writeString(String str, uLong sector, uInt offset);  // write string or String(number) to SD
  DRESULT writeRaw (const uChar* buff, uLong sector, uInt offsetInSector, uInt bytesToWrite);  // write a raw block of data to SD
  DRESULT readRaw (uChar* buff, uLong sector, uInt offsetInSector, uInt bytesToRead);  // read a raw block data from SD
};

#endif
