// www.kendziorra.nl (c) 2014
// GL5528: -10C <> 125C, 19,5mV/1C, 0C = 400mV
//  1- read sensor average voltage
//  2- calc lux (from repeated reading)
//  3- add compensation and return lux
// return -9999 if PIN not set
/* usage:
setup()
{
  Lux.begin(A-PIN);  // sensor pin  
  Lux.setLuxComp(float);
}

loop()
{
  lux = Lux.getLux();  // get lux from GL5528, -9999 ij luxPin not set
  Serial.println(lux);
  delay(1000);
}
*/

#include "GL5528lux.h"

boolean initLux = false;

void LuxClass::setLuxComp(byte LuxComp)
{ 
  _comp = LuxComp;
}

void LuxClass::begin(byte luxPin)
{ 
  _pin = luxPin;
  initLux = true;
  getLux();
}

int LuxClass::getLux()
{ 
  if (!initLux) return -9999;  // luxPin not set
  float volt = 0.0, luxSum = 0.0, lux = 0.0;
  int repeats = 5;
   float r;
   byte i = 0;
  while (i < repeats)
  {
    volt = analogRead(_pin) * 5.0 / 1024.0;
    r = (1.0 * 5.0) / volt - 1.0;  // 1.0 = weerstand van 1Kohm
    lux = 255.84 * pow(r, -10.0/9.0) + _comp;
    if ((lux - _lux) < 0.2 || (lux - _lux) > -0.2)
    {
       luxSum += lux;
      _lux = lux;
      i++;
    }
  }
  return (int)((luxSum / repeats + _comp + 0.05) * 10.0) / 10.0;
}

LuxClass Lux;

