// www.kendziorra.nl (c) 2014
// MCP9701: -10C <> 125C, 19,5mV/1C, 0C = 400mV
//  1- read sensor average voltage
//  2- calc temperature (from repeated reading)
//  3- add compensation and return temperature
// return -99.99 if PIN not set
/* usage:
setup()
{
  Temp.begin(A-PIN);  // sensor pin  
  Temp.setTempComp(float);
}

loop()
{
  temp = Temp.getTemp();  // get temperature from MCP9701, -99.99 if tempPin not set
  Serial.println(temp);
  delay(1000);
}
*/

#include "MCP9701temp.h"

boolean initTemp = false;

void TempClass::setTempComp(float tempComp)
{ 
  _comp = tempComp;
}

void TempClass::begin(byte tempPin)
{ 
  _pin = tempPin;
  initTemp = true;
  getTemp();
}

float TempClass::getTemp()
{ 
  if (!initTemp) return -99.99;  // tempPin not set
  float volt = 0.0, tempSum = 0.0, temp = 0.0;
  int repeats = 5;
  byte i = 0;
  while (i < repeats)
  {
    volt = analogRead(_pin) * 5.0 / 1024.0;
    temp = ((volt - 0.4) / 0.0195);
    if ((temp - _temp) < 0.2 || (temp - _temp) > -0.2)
    {
       tempSum += temp;
      _temp = temp;
      i++;
    }
  }
  return (int)((tempSum / repeats + _comp + 0.05) * 10.0) / 10.0;
}

TempClass Temp; 

