
// ##### read readme.h #####

#include <Streaming.h>  // used for Serial OUT
#include <Time.h>
#include "TimeDate.h"

//### SET Time zone and DST/Summer time offsets ###
// EU Summer Time (DST) (explanation in readme.h): 
//   start at the last sunday of Mar. on 02:00, offset +1h (add dstOffset to local time)
//   ends at the last sunday of Oct. on  03:00, offset -1h (subtract dstOffset from local time)
byte dstStartMonth =  3, dstStartMonthDay = 0, dstStartWeek = 0, dstStartWeekDay = 1, dstStartHour = 2,  // DST start date rules
     dstEndMonth   = 10, dstEndMonthDay   = 0,   dstEndWeek = 0,   dstEndWeekDay = 1,   dstEndHour = 3;  // DST end date rules
long utcOffset = 3600;  // UTC/GMT offset in seconds (EU = UTC+1)
int dstOffset = 3600;  // DST/Summer Time offset in seconds (EU = EU time zone +1)
unsigned long testTime;

/* ############### In this example ###############
   ##### The clock is set to fictional times #####
   #####   Set the time always as UTC/GMT    #####
   ############################################### */

void setup()
{
  Serial.begin(9600);
  delay(2000);
  Serial << F("DEMO Universal usable DST/and Summer Time Library\r\n\r\n");
  setTime(16, 34, 0, 21, 5, 14);  // UTC 
  testTime = now();
  Serial << F("  Europe Summer Time, UTC+1, DST+1, Su last week Mar and Oct\r\n"
            "===============================================================\r\n");
  TD.initDst(dstStartMonth, dstStartMonthDay, dstStartWeek, dstStartWeekDay, dstStartHour,  // DST start date rules
             dstEndMonth,   dstEndMonthDay,   dstEndWeek,   dstEndWeekDay,   dstEndHour,    // DST end date rules
             utcOffset, dstOffset);  // time zone and DST in seconds
  showDst(now(), true);  // DST/Summer Time?

  // ## test if it works properly with summer in the middle of the year
  testDst1();  // good job?
  // ## setTime() is changed, set time to UTC Time
  setTime(testTime); 

  // show the universal mode
  Serial << F("\r\n   USA, UTC-5, DST+1, Su second week Mar, Su first week Nov\r\n"
                "===============================================================\r\n");
  utcOffset = 5.0 * -3600.0; dstOffset = 3600; dstStartHour = 2; dstEndHour = 2;
  TD.initDst(3, 0, 2, 1, dstStartHour,  // Su. second week of Mar.
            11, 0, 1, 1, dstEndHour,  // Su. first week of Nov.
            utcOffset, dstOffset);  // UTC-4, DST offset 
  showDst(now(), true);

  Serial << F("          FIXED DATES, UTC+0, DST+1, 1 Mar and 1 Oct\r\n"
            "===============================================================\r\n");
  utcOffset = 0; dstOffset = 3600; dstStartHour = 2; dstEndHour = 2;
  TD.initDst(3, 1, 0, 1, dstStartHour,  // FIXED to 1 Mar.
            10, 1, 0, 1, dstEndHour,  // FIXED to 1 Oct.
            utcOffset, dstOffset);  // UTC+0, DST offset
  showDst(now(), true);

  Serial << F("Lord Howe Island, UTC+10:30, DST+0:30, Su first week Oct and Apr\r\n"
            "================================================================\r\n");
  utcOffset = 10.0 * 3600.0 + 30.0 * 60.0; dstOffset = 1800; dstStartHour = 2; dstEndHour = 2;
  TD.initDst(10, 0, 1, 1, dstStartHour,  // Su. first week of Oct. 
             4, 0, 1, 1, dstEndHour,  // Su. first week of Apr.
             utcOffset, dstOffset);  // UTC+10:30, DST offset
  showDst(now(), true);

  // ## test if it works properly with winter in the middle of the year
  testDst2();  // good job?
  // ## setTime() is changed, set time to Local Time
  setTime(testTime); 
}

void loop()
{}

void testDst1()  // test EU Summer Time with one second
{
  Serial << F("\tEurope Summer Time TEST\r\n\t=======================\r\n");
  // we need UTC time
  setTime(TD.calcTimeStamp(12, 0, 0, 1, 1, 14) - utcOffset);  // Time: 1 Jan. 2014 12:00:00
  showDst(now(), false);
  setTime(TD.calcTimeStamp(2, 0, 0, 30, 3, 14) - utcOffset);  // Start Summer Time: Su. 30 Mar. 2014 02:00:00
  showDst(now() - 1, false);
  showDst(now(), false);
  setTime(TD.calcTimeStamp(12, 0, 0, 1, 7, 14) - utcOffset - dstOffset);  // Time: 1 Jul. 2014 12:00:00
  showDst(now(), false);
  setTime(TD.calcTimeStamp(3, 0, 0, 26, 10, 14) - utcOffset - dstOffset);  // End Summer Time: Su. 26 Oct. 2014 03:00:00
  showDst(now() - 1, false);
  showDst(now(), false);
  setTime(TD.calcTimeStamp(12, 0, 0, 31, 12, 14) - utcOffset);  // Time: 31 Dec. 2014 12:00:00
  showDst(now(), false);
}

void testDst2()  // test Summer Time with one second
{
  Serial << F("\tLord Howe Island DST TEST\r\n\t=========================\r\n");
  // we need UTC time
  setTime(TD.calcTimeStamp(12, 0, 0, 1, 1, 14) - utcOffset - dstOffset);  // Time: 1 Jan. 2014 12:00:00
  showDst(now(), false);
  setTime(TD.calcTimeStamp(2, 0, 0, 6, 4, 14) - utcOffset - dstOffset);  // End Summer Time: Su. 6 Apr. 2014 02:00:00
  showDst(now() - 1, false);
  showDst(now(), false);
  setTime(TD.calcTimeStamp(12, 0, 0, 1, 7, 14) - utcOffset);  // Time: 1 Jul. 2014 12:00:00
  showDst(now(), false);
  setTime(TD.calcTimeStamp(2, 0, 0, 5, 10, 14) - utcOffset);  // 
  showDst(now() - 1, false);
  showDst(now(), false);
  setTime(TD.calcTimeStamp(12, 0, 0, 31, 12, 14) - utcOffset);  // Time: 31 Dec. 2014 12:00:00
  showDst(now(), false);
}

void showDst(unsigned long utcTime, boolean f)
{
  // 'localTimeOffset': isDst, startDst and endDst are also calculated  
  Serial << formatDate(utcTime) << "-UTC / " << formatDate(utcTime + TD.localTimeOffset(utcTime)) << "-Loc";
  if (TD.isDst) Serial << F(", DST: true\r\n");
  else Serial << F(", DST: false\r\n");
 
  if (f)
  {
    String str[] = {"DST start at ", "DST  ends at ", "\r\n\tAdd: ", "sec to correct the Local Time\r\n"};
    if (TD.startDst < TD.endDst)
    {
      Serial << str[0] << formatDate(TD.startDst) << str[2] << " " << dstOffset << str[3];
      Serial << str[1] << formatDate(TD.endDst) << str[2] << -dstOffset << str[3];
    }
    else
    {
      Serial << str[1] << formatDate(TD.endDst) << str[2] << " " << -dstOffset << str[3];
      Serial << str[0] << formatDate(TD.startDst) << str[2] << dstOffset << str[3];
    }
    Serial << endl;
  }
}

String formatDate(unsigned long t) // IN: UNIX time stamp - OUT: date notation: dd d mmm yyyy hh:MM:ss
{
  return TD.formatDate(t, "DD dd MMM yyyy") + " " + TD.formatTime(t, false, false);
}

