/* Time and Date Libraries                                   *
 * (c) 2014 by Ewald Kendziorra sr, www.kendziorra.nl        *
 * Free to use, modify and distribute provided               *
 *    that this readme header file is included               *
 * The use of this software is for your own risk             *
 /                                                           /
 / My collection of Date and Time libraries:                 /
 /   # TimeHelpers: extra Date and Time functions            /
 /   # Dst        : Universal DST / Summer Time calculation  /
 /   # Sun        : Sunrise and Sunset calculation           /
 /   # Ntp        : NTP server time sync  (Spark Core)       /
 /                                                           /
 / Use my Serial Monitor 8n1 for Arduino, Spark Core, ...    /
 /              http://www.kendziorra.nl/                   */

// # download Streaming.h from http://arduiniana.org/libraries/streaming/

//==============================EXAMPLE CODE=============================
// Serial time and date input in 'TimeDateDemo.ino'
// Universal DST/Summer time in 'DstSummerTime.ino'
// Format date and time in 'format.ino'
// All together, included Sunrise and Sunset, in 'TimeDateDemo.ino'
// Arduino: if no RTC/NTP/... use softTime (in 'TimeDateDemo.ino') and 
//    use the Serial input to set/correct the date and or time 
//=======================================================================
// #### use the prefix TD.<function()> TD.<global variable> #####
//                     ===          ===                       

/* ##### TimeHelpers functions ##### */

/* calcTimeStamp   get UNIX time_t Time Stamp (for Spark Core)
   leapYear        leap year?
   yearDay         year days from timestamp
   yearDay         year days from date numbers
   twoDigits       format a char or byte to two digits "00"-"99" or " 0"-" 9"-"99"
   dayName         get the name of the day
   monthName       get the name of the month
   formatTime      format a time string (see bellow)
   formatDate      format a date string (sie bellow)
 /
 / formatTime
 / boolean sec: "00:00:00"; !sec: "00:00"
 / boolean  AM: "00:00(:00) AM/PM"; !AM "00:00(:00)" 
 / example: time in t is hour(14), minute(5), second(6) 
 / ========
 /    sec,  AM -> "02:05:06 PM"
 /    sec, !AM -> "14:05:06"
 /   !sec,  AM -> "02:05 PM"
 /   !sec, !AM -> "14:05"
 *
 / formatDate
 / this function replace chars with a date part
 / the parts are not restricted to a particular area in the char string 'format'
 / delimiters are required, allowable delimiters: ' ', '-', '/'
 / "xd"   -> 1-31       day
 / "dd"   -> 01-31      day
 / "xm"   -> 1-12       month
 / "mm"   -> 01-12      month
 / "yyyy" -> 1970-2099  year
 / "DD"   -> Su-Za      short day name
 / "MMM"  -> Jan-Dec    short month name
 / example: date in time 't' is day(4), month(5), year(2014)
 / ========
 /   "DD dd-mm-yyyy"  -> "Wo 04-05-2014"
 /   "DD xd MMM yyyy" -> "Wo 4 May 2014"
 /   "xm/xd/yyyy"     -> "5/4/2015"                    */
 */

//=======================================================================
/* ##### Dst example data ##### */

/* DST/Summer time:
/ * ... stands for startMonth, endMonth, startDay, endDay, ......
/ * ===            =====       ===       =====     ===
/ * Variable date with ...Month, ...Week (1-4 or last) and ...WeekDay 
/ * Fixed date with ...Month and ...MonthDay
/ if (...Month == 0)                        DST/Summer Time will not be used
/ if (...MonthDay == 0 && ...WeekDay == 0)  DST/Summer Time will not be used
/ if (...MonthDay != 0)                     then ...WeekDay is ignored
/ if (...WeekDay != 0 && ...Week != 0)      then ...Week is 1, 2, 3 or 4
/ if (...WeekDay != 0 && ...Week == 0)      it's the last week of the month!!
/ Sunday is day 1 of the week, Saturday is day 7 of the week 
/
/ TD.initDst(dstStartMonth, dstStartMonthDay, dstStartWeek, dstStartWeekDay, dstStartHour,  // DST start date rules
/            dstEndMonth,   dstEndMonthDay,   dstEndWeek,   dstEndWeekDay,   dstEndHour,    // DST end date rules
/            utcOffset, dstOffset);  // time zone and DST in seconds
/
/ TD.localTimeOffset(UTCtimestamp) calculate the local time timestamp globals 'TD.startDst' and 'TD.endDst' and (re)set flag 'TD.isDst'
/ Serial.print("DST start on: "); Serial.print(TD.startDst); Serial.print("  DST end on: "); Serial.print(TD.endDst);
/ print is easier with Streaming.h ;)
/ Serial << "DST start on: " << TD.startDst) << " - DST ends on: " << TD.endDst << endl;
/
/ ##### example data for 'Europe', 'US', 'FIXED DATE' and 'LORD HOWE ISLAND'(;>))  */

/* EU Summer Time
 * ============== 
 * start: sunday, last week of mrt, 02:00(UTC+1) -> 03:00(UTC+2) +3600s(1h) 
 * end:   sunday, last week of oct, 03:00(UTC+2) -> 02:00(UTC+1) -3600s(1h) */
//  setTime(12, 0, 0, 30, 4, 14); // hh:MM:ss dd-mm-yy(yy) change this to correct Local Time or use RTC or ...
//  TD.initDst(3, 0, 0, 1, 2,          // init Dst whith DST start data | ...Month, ...MonthDay, ...Week, ...WeekDay, ...Hour
//            10, 0, 0, 1, 3,          // init Dst whith DST end data | ...Month, ...MonthDay, ...Week, ...WeekDay, ...Hour
//            3600, 3600);             // UTC/GMT offset in seconds (EU = UTC+1), DST/Summer Time offset in seconds (EU = EU time zone +1)

/* USA DST
 * ======= */
//  TD.initDst(3, 0, 2, 1, 2,      // Su. second week of Mar.
//            11, 0, 1, 1, 2,      // Su. first week of Nov.
//            5.0 * -3600.0, 3600); // UTC-5, DST+1

/* FIXED DATES DST
 * =============== */
//  TD.initDst(3, 1, 0, 1, 2, // FIXED to 1 Mar.
//            10, 1, 0, 1, 2, // FIXED to 1 Oct.
//            0, 3600);       // UTC+0, DST+1 

/* Lord Howe Island DST   (;>))
 * ==================== */
//  TD.initDst(10, 0, 1, 1, 2,   // Su. first week of Oct. 
//              4, 0, 1, 1, 2);  // Su. first week of Apr.
//             10.0 * 3600.0 + 30.0 * 60.0, 1800); //UTC+10:30, DST+0:30

//=======================================================================
/* ##### Sun example data ##### */

// //### SET YOUR Location, Time zone and DST/Summer time offsets ###
// // the Centre of the Netherlands, timezone UTC/GMT+1, lat 52.0, lon 5.0
// const float latitude = 52.0, longitude = 5.0;
// const long utcOffset = 3600;  // offset to Local Time in seconds
// const int dstOffset= 3600;  // DST/Summer Time offset in seconds 

// // in setup:
// TD.initSun(latitude, longitude, dst ? UTC + DST : UTC);  // init Sun once and when DST/Summer Time is changed
//
// // in your function:
// //if DST/Summer Time is changed then init again
// TD.initSun(latitude, longitude, dst ? utcOffset + dstOffset : utcOffset);  // init Sun when DST/Summer Time is changed
// // calculate sunrie and sunset
// TD.CalcSunTimes(Time.day(), Time.month(), Time.year(), dstoffset);  // calculate Sun
// // the sunrise and sunset time now():
// Serial.print("Sunrise: "); Serial.print(TD.sunRiseHour); Serial.print(":"); Serial.printl(TD.sunRiseMinute );
// Serial.print("Sunset : "); Serial.print(TD.sunSetHour);  Serial.print(":"); Serial.printl(TD.sunSetMinute );
// // print is easier with Streaming.h ;)
// Serial << "Sunrise: " << TD.sunRiseHour << ":" << TD.sunRiseMinute << 
//           "Sunset : " << TD.sunSetHour  << ":" << TD.sunSetMinute << endl;

//=======================================================================
/* #### NTP Time Server example #### */

// const unsigned int ntpPort = 123;  // (change it) local UDP port for Time Server
// const char *ntpServer[] = {"0.nl.pool.ntp.org", "1.nl.pool.ntp.org",  // (change it) four NTP servers for use in the Netherlands
//                            "2.nl.pool.ntp.org", "3.nl.pool.ntp.org"};
// const unsigned int syncOffset = 1800;  // time sync after each n seconds; small value = more precision
// unsigned long isSyncTime;  // contains the next syncTime timestamp

// void loop()
// {
//   /*####################################################################
//     SPARK CORE incorrectly Unix 'time_t' timestamp integer type
//     The correct type is an 'unsigned long' or 'unsigned int' (2^32 - 1)
//     ####################################################################*/
//   if (isSyncTime <= (unsigned long)Time.now()) syncTime();  // time to sync the time?
//                   //---------------
// }

// void syncTime()  // sync the time with NTP
// { 
//   for (byte i = 0; i <= 3; i++)  // using four time servers
//     if (TD.timeSync(ntpServer[0], ntpPort))
//       if (!TD.timeIsSynced)
//       {
//         isSyncTime += syncOffset;  // set the next sync timestamp (is used in loop())
//         usedServer = i;  // ##### is only needed for this example
//         break;
//       }
// }




