
//##### read readme.h #####

// ##### for using a RTC, NTP or other different kind of clock module then add the associated library
#define SOFTTIME  // if there is no RTC/NTP/... clock to sync the time then we can use the SoftTime time correction
//                      SoftTime is not a library function, the function call is in loop()

#include <Time.h>
#include <Streaming.h>
#include "TimeDate.h"

#ifdef SOFTTIME   //    to correct the time periodical
const unsigned int softTimeOffset = 3600;  // time correctiom after each n seconds
const int softTimeCorrection = -1; // correction in seconds, value should be determined by experimental
                                  //   can be a positive or negative value
unsigned long nextCorrectTime;  // contains the next softTime set in seconds
#endif  // SOFTTIME

//### SET YOUR Location, Time zone and DST/Summer time offsets ###
// the Centre of the Netherlands, timezone UTC/GMT+1, lat 52.0, lon 5.0
// EU Summer Time (DST) (explanation in readme.h): 
//   start at the last sunday of Mar. on 02:00, offset +1h (add dstOffset to local time)
//   ends at the last sunday of Oct. on  03:00, offset -1h (subtract dstOffset from local time)
const float latitude = 52.0, longitude = 5.0;  // in deximals, used for Surise and Sunset
const byte dstStartMonth =  3, dstStartMonthDay = 0, dstStartWeek = 0, dstStartWeekDay = 1, dstStartHour = 2,  // DST start date rules
           dstEndMonth   = 10, dstEndMonthDay   = 0,   dstEndWeek = 0,   dstEndWeekDay = 1,   dstEndHour = 3;  // DST end date rules
const long utcOffset = 3600;  // UTC/GMT offset in seconds (EU = UTC+1)
const int dstOffset = 3600;  // DST/Summer Time offset in seconds (EU = EU time zone +1)
const char* location = "the Centre of the Netherlands";  // ##### is only needed for this example

unsigned long showTime;  // to show the time every one sec

void showDateTime (unsigned long);
byte digitsToByte (char*);
char* subChars (char*, byte, byte);

void setup()
{
  Serial.begin(9600);
  delay(2000);  // wait n sec for loading USB-COM driver
  Serial << F("\tDemo date and time library with\r\nUniversal usable DST/summer time, every DST rule is possible\r\n"
              "Sunrise and sunset\r\nDate and time formatting with short day and month names\r\n\r\n");
  setTime(TD.calcTimeStamp(14, 50, 0, 24, 5, 14));  // basis start for NTP (clock time is always in UTC/GMT) 
#ifdef SOFTTIME
  nextCorrectTime = now() + softTimeOffset;
#endif
  TD.initDst(dstStartMonth, dstStartMonthDay, dstStartWeek, dstStartWeekDay, dstStartHour,  // DST start date rules
             dstEndMonth,   dstEndMonthDay,   dstEndWeek,   dstEndWeekDay,   dstEndHour,    // DST end date rules
             utcOffset, dstOffset);  // time zone and DST in seconds
  TD.initSun(latitude, longitude, utcOffset);  // init Sun with location in decimals and time zone in seconds
  showDateTime(now());
}

void loop()
{
#ifdef SOFTTIME  // adjust time 
  if (nextCorrectTime <= now()) correctTime();
#else 
  // add code here to sync the time with a RTC/NTP/... library
#endif
  // show time every five sec 
  if (showTime <= now())
  {
    showTime = now() + 1;  // show the time every one sec
    unsigned long localTime = now() + TD.localTimeOffset(now());
    Serial << "\f";  // with my Serial Monitor 8n1: if \f checked then overwrite the previous time
    Serial << TD.formatDate(localTime, "DD xd MMM yyyy") << " "
            << TD.formatTime(localTime, true, false) << endl;
  }
  delay(50); 
}

#ifdef SOFTTIME  // adjust time  
void correctTime()
{
  setTime(now() + softTimeCorrection);  // adjust time with nn seconds
  nextCorrectTime = now() + softTimeOffset;
  Serial << "Time correction with " << softTimeCorrection << "s\r\n"; 
}
#endif

void showDateTime(unsigned long utcTime)  // time in UTC/GMT !!!
{
  unsigned long localTime = utcTime + TD.localTimeOffset(utcTime);  // 'localTimeOffset': isDst, startDst and endDst are also calculated  
  TD.CalcSunTimes(day(utcTime), month(utcTime), year(utcTime), dstOffset);  // calculate SunRise and SunSet in local time
  Serial << 
//    F("\r\n\tUsed Time server : ") << (TD.timeIsSynced ? ntpServer[usedServer] : "still no result")<<
//    F("\r\n\tTime is synced   : ") << (TD.timeIsSynced ? "true" : "false") <<
    F("\r\n\t------Local------") << 
    F("\r\n\tlocation         : ") << location << 
    F("\r\n\tlatitude         : ") << latitude << 
    F("\r\n\tlongitude        : ") << longitude <<
    F("\r\n\ttime zone        : ") << (utcOffset / 3600) <<
    F("\r\n\tDST / Summer time: ") << (TD.isDst ? "true" : "false") <<
    F("\r\n\ttime             : ") << TD.formatDate(localTime, "dd-mm-yyyy ") << TD.formatTime(localTime, true, false) << 
    F("\r\n\tUnix timestamp   : ") << localTime <<  
    F("\r\n\tleap year        : ") << (TD.leapYear(year(localTime)) ? "true" : "false") << 
    F("\r\n\tyear day         : ") << TD.yearDay(localTime) << 
    F("\r\n\tDST Start        : ") << TD.formatDate(TD.startDst, "DD dd MMM ") << TD.formatTime(TD.startDst, false, false) << "  add: +" << (dstOffset / 3600) << "h" << 
    F("\r\n\tDST Ends         : ") << TD.formatDate(TD.endDst, "DD dd MMM ") << TD.formatTime(TD.endDst, false, false) << "  add: -" << (dstOffset / 3600) << "h" <<
    F("\r\n\tsunrise          : ") << TD.formatTime(TD.calcTimeStamp(TD.sunRiseHour, TD.sunRiseMinute, 0, 1, 1, year(localTime)), false, false) << 
    F("\r\n\tsunset           : ") << TD.formatTime(TD.calcTimeStamp(TD.sunSetHour, TD.sunSetMinute, 0, 1, 1, year(localTime)), false, false) << 
    F("\r\n\t-----UTC/GMT-----") << 
    F("\r\n\ttime             : ") << TD.formatDate(utcTime, "dd-mm-yyyy ") << TD.formatTime(utcTime, true, false) << 
    F("\r\n\tUnix timestamp   : ") << utcTime << 
    F("\r\n\tDST Start        : ") << TD.formatDate(TD.startDst - dstOffset, "DD dd MMM ") << TD.formatTime(TD.startDst - dstOffset, false, false) << 
    F("\r\n\tDST Ends         : ") << TD.formatDate(TD.endDst - utcOffset - dstOffset, "DD dd MMM ") << TD.formatTime(TD.endDst - utcOffset - dstOffset, false, false) << endl;
}

// ##### SERIAL TIME INPUT #####
String buffer = "";
void serialEvent()  // Serial interrupt event to set time and date as UTC/GMT
{ 
  // set Date and Time, format: time hh:mm:ss, date dd-mm-yy
  // example: 1 May 2014 9:12:8    Send date: 01-05-14    Send time: 09:12:08
  char inChar = (char)Serial.read();
  buffer += inChar;
  if (buffer.length() == 8)
  {
    inputTime(buffer);
    while (Serial.available()) Serial.read();  // empty serial buffer
    buffer = "";
  }
}

void inputTime(String buff)
{
  unsigned long t;
  byte test = 0;  // time or date?
  if (buff.substring(2, 3) == ":" && buff.substring(5, 6) == ":") test = 1;  // time
  else if (buff.substring(2, 3) == "-" && buff.substring(5, 6) == "-") test = 2;  // date
  if (test)  // test on valid time/date format
      for (int i = 0; i < 8; i += 3)
    {
      if (buff.substring(i, i + 1) < "0" || buff.substring(i, i + 1) > "9") test = 0; 
      if (buff.substring(i + 1, i + 2) < "0" || buff.substring(i + 1, i + 2) > "9") test = 0; 
    }
  if (test == 1)  // change time
  {  // hh:mm:ss
    
    t = TD.calcTimeStamp(buff.substring(0, 2).toInt(), 
    buff.substring(3, 5).toInt(),
    buff.substring(6, 8).toInt(),
    day(), month(), year());
    setTime(t - TD.localTimeOffset(t));
  }
  else if (test == 2)  // change date
  {  // DD-MM-YY
    t = TD.calcTimeStamp(hour(), minute(), second(),
    buff.substring(0, 2).toInt(), 
    buff.substring(3, 5).toInt(),
    buff.substring(6, 8).toInt());
    setTime(t);
  }
  if (test)
  {
    showDateTime(now());
    showTime = now();
  }
}

