
//##### read readme.h #####

#ifndef TIMEDATE_h
#define TIMEDATE_h

#if defined(ARDUINO)
  #if ARDUINO >= 100
	#include "Arduino.h"
  #else
	#include "WProgram.h"
  #endif
#elif defined(SPARK_CORE)
	#include "application.h"
#endif

#include <math.h>

//#define PI 3.14159265
//#define DEG_TO_RAD 0.01745329
//#define RAD_TO_DEG 57.2957786

// date constants
const unsigned long oneDay = 86400;  // 24 * 60 * 60  one day in sec 
const unsigned long oneWeek = 604800;  // oneDay * 7  one week in sec

class TimeDate{

public:
  // ##### helpers #####
  unsigned long calcTimeStamp (byte, byte, byte, byte, byte, int);  // get UNIX time_t Time Stamp from hh:mm:ss DD-MM-YY
  boolean leapYear (int);  // leap year?
  int yearDay (unsigned long);  // year days from timestamp
  int yearDay (byte, byte, int);  // year days from date numbers
  String twoDigits (byte, boolean);  // format a char or byte n (0-99) to char* two digits "00"-"99", or if spatie " 0"-"99"
  String dayName (byte);  // get the name of the day
  String monthName (byte);  // get the name of the month
  String formatTime(unsigned long, boolean, boolean);  // format a time char string 
  String formatDate(unsigned long, String);  //format a date char string 
  // ##### DST #####
  unsigned long startDst, endDst;  // 'localTimeOffset' set DST/Summer start and end time (UNIX time_t time stamp) 
  boolean isDst;  // 'localTimeOffset' set this flag if DST/Summer time
  void initDst (byte, byte, byte, byte, byte,  // DST start date rules
                byte, byte, byte, byte, byte,  // DST end date rules
                long, int);                     // UTC and DST offset
  long localTimeOffset(unsigned long);  // it calls 'calcDst' and determine time offset: time zone or time zone + DST offset
  // ##### SUN #####
  float sunRiseDec, sunSetDec;  // rise and set as DEC time (12.75)
  byte sunRiseHour, sunRiseMinute, sunSetHour, sunSetMinute;  // sunrise and suset as time (12:45)
  void initSun (float, float, int);  // init Sun with location and time zone in seconds
  void CalcSunTimes (byte, byte, int, unsigned int);  // calculate SunRise and SunSet in local time 
private:
  // ##### DST #####
  byte startMonth, startMonthDay, startWeek, startWeekDay, startHour,
       endMonth,   endMonthDay,   endWeek,   endWeekDay,   endHour;
  long utcOffset;
  int dstOffset;
  boolean calcDst (unsigned long);  // DST/Summer Time?
  unsigned long findDstDay (int, byte, byte, byte, byte, byte);
  // ##### SUN #####
  float lat, lon, timeZone;
  float rangeAngle (float);  
};

extern TimeDate TD;

#endif
