﻿using System;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Linq;
//using System.Text;
//using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;
//using System.Diagnostics;


namespace HomeSwitch
{
  public partial class homeSwitch : Form
  {
    public delegate void SerialData(string text);  //voorkom threading errors tijdens ontvangen van data
    private string iniFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\HomeSwitch.dat"; // opslaan na update
    private TimeSpan rtc; // verschil in PC-tijd en RTC
    private int yyyy, MM, dd, hh, mm, ss; // RTC

    public homeSwitch()
    { //init
      InitializeComponent();
    }

    private void homeSwitch_FormClosed(object sender, FormClosedEventArgs e)
    { // afsluiten
      try
      {
        if (serialPort1.IsOpen)
        {
          serialPort1.DiscardInBuffer();
          serialPort1.Close();
        }
      }
      catch (Exception) { }
    }

    private void homeSwitch_Load(object sender, EventArgs e)
    { // openen
      string[] myPort = SerialPort.GetPortNames();
      //haal Compoorten op
      if (myPort.Length == 0)
      {
        MessageBox.Show("Geen COMpoort gevonden");
        Close();
        return;
      }
      for (int i = 0; i < myPort.Length; i++)
      {
        cmbPoort.Items.Add(myPort[i]);
      }
      cmbPoort.SelectedIndex = 0;
      cmbBaud.SelectedIndex = 0;
    }

    private void btnOpen_Click(object sender, EventArgs e)
    { // poort openen en controls instellen
      try
      {
        serialPort1.PortName = cmbPoort.Text;
        serialPort1.BaudRate = Convert.ToInt32(cmbBaud.Text);
        serialPort1.Open();
      }
      catch (Exception)
      {
        MessageBox.Show("Helaas: kan poort niet openen");
        return;
      }
      if (serialPort1.IsOpen)
      {
        cmbPoort.Enabled = false;
        cmbBaud.Enabled = false;
        btnOpen.Enabled = false;
        btnSluit.Enabled = true;
        btnLaden.Enabled = true;
        btnUpdate.Enabled = true;
        grbLicht.Enabled = true;
        serialPort1.DiscardInBuffer();
        TransmitText("K1\n");		// haal RTC-DatumTijd en alle gegevens op
      }
    }

    private void btnSluit_Click(object sender, EventArgs e)
    { // poort sluiten en controls resetten
      if (serialPort1.IsOpen) serialPort1.Close();
      btnOpen.Enabled = true;
      btnSluit.Enabled = false;
      btnLaden.Enabled = false;
      btnUpdate.Enabled = false;
      grbLicht.Enabled = false;
      cmbCode.SelectedIndex = -1;
      cmbAan.SelectedIndex = -1;
      cmbUit.SelectedIndex = -1;
      cmbOffset.SelectedIndex = -1;
      chbOchtend.Checked = false;
      chbAvond.Checked = false;
      lblOp.Text = "";
      lblOnder.Text = "";
      lblRTC.Text = "";
      for (int i = 1; i <= 16; i++)
      {  // vink alle lampen en DIM uit
        CheckBox checkbox1 = (CheckBox)grbLicht.Controls["chb" + i.ToString()];
        checkbox1.Checked = false;
        CheckBox checkbox2 = (CheckBox)grbLicht.Controls["chbD" + i.ToString()];
        checkbox2.Checked = false;
      }
    }

    private void cmbUpdate_Click(object sender, EventArgs e)
    { // Arduino updaten
      if (this.serialPort1.IsOpen)
      {
        TransmitText("U01" + System.DateTime.Now.ToString("dd") + "\n");
        TransmitText("U02" + System.DateTime.Now.ToString("MM") + "\n");
        TransmitText("U03" + System.DateTime.Now.ToString("yy") + "\n");
        TransmitText("U04" + System.DateTime.Now.ToString("HH") + "\n");
        TransmitText("U05" + System.DateTime.Now.ToString("mm") + "\n");
        TransmitText("U06" + System.DateTime.Now.ToString("ss") + "\n");
        String s = "";
        for (int i = 1; i <= 16; i++)
        {  //  lampen Aan of Uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chb" + i.ToString()];
          s += checkbox.Checked ? "1" : "0";
        }
        string bewaar = s + Environment.NewLine;
        TransmitText("U07" + s + "\n");
        s = "";
        for (int i = 1; i <= 16; i++)
        {  // isDim Aan of Uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chbD" + i.ToString()];
          s += checkbox.Checked ? "1" : "0";
        }
        bewaar += s + Environment.NewLine;
        TransmitText("U08" + s + "\n");
        TransmitText("U09" + cmbOffset.Text + "\n");
        TransmitText("U10" + (chbOchtend.Checked ? "1" : "0") + "\n");
        TransmitText("U11" + (chbAvond.Checked ? "1" : "0") + "\n");
        TransmitText("U12" + cmbAan.Text.Substring(0, 2) + "\n");
        TransmitText("U13" + cmbAan.Text.Substring(3) + "\n");
        TransmitText("U14" + cmbUit.Text.Substring(0, 2) + "\n");
        TransmitText("U15" + cmbUit.Text.Substring(3) + "\n");
        TransmitText("U16" + cmbCode.Text + "\n");
        bewaar += cmbOffset.Text + Environment.NewLine +
          (chbOchtend.Checked ? "1" : "0") + Environment.NewLine +
          (chbAvond.Checked ? "1" : "0") + Environment.NewLine +
          cmbAan.Text.Substring(0, 2) + Environment.NewLine +
          cmbAan.Text.Substring(3) + Environment.NewLine +
          cmbUit.Text.Substring(0, 2) + Environment.NewLine +
          cmbUit.Text.Substring(3) + Environment.NewLine +
          cmbCode.Text + Environment.NewLine;
        using (StreamWriter sw = new StreamWriter(iniFile)) sw.WriteLine(bewaar);
      }
    }

    private void cmbLaden_Click(object sender, EventArgs e)
    { // opgeslagen gegevens ophalen en instellen
      if (File.Exists(iniFile))
      {
        try
        {
        string s = "";
        using (StreamReader sr = new StreamReader(iniFile)) { s = sr.ReadToEnd(); }
        string[] ar = s.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.None);
        for (int i = 1; i <= 16; i++)
        {  // vink alle lampen aan/uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chb" + i.ToString()];
          checkbox.Checked = (ar[0].Substring(i - 1, 1).ToString() == "1");
        }
        for (int i = 1; i <= 16; i++)
        {  // vink alle DIM aan/uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chbD" + i.ToString()];
          checkbox.Checked = (ar[1].Substring(i - 1, 1).ToString() == "1");
        }
        cmbOffset.Text = ar[2];
        chbOchtend.Checked = ar[3] == "1";
        chbAvond.Checked = ar[4] == "1";
        cmbAan.Text = ar[5];
        cmbAan.Text = ar[6];
        cmbUit.Text = ar[7];
        cmbUit.Text = ar[8];
        cmbCode.Text = ar[9];
        }
        catch (Exception)
        {
         MessageBox.Show("Helaa: HomeSwitch.dat fout.\nWordt bij 'Update Arduino' opnieuw aangemaakt");
        }
      }
      else
      {
        MessageBox.Show("HomeSwitch.dat niet gevonden.\nWordt bij 'Update Arduino' aangemaakt");
      }
    }

    private void timer1_Tick(object sender, EventArgs e)
    {
      // datum en tijd bijhouden (timer1 staat op 30 sec. (30000ms))
      DateTime nu = DateTime.Now;
      if (this.lblRTC.Text != "")
      { // alleen als RTC-tijd er is
        DateTime dt = nu + rtc; // verreken de PC-tijd met het verschil (TimeSpan rtc)
        this.lblRTC.Text = dt.ToString("dd-MM-yyyy HH:mm");
      }
    }

    private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
    { // interrupt: data aanwezig in poortbuffer
      string s = "";
      s = serialPort1.ReadLine();	// als ascii regel ontvangen
      BeginInvoke(new SerialData(serialData), new object[] { s }); // in deze thread houden
    }

    private void serialData(string s)
    { // data van Arduino uitlezen
      int j = s.IndexOf("K");
      string k = s.Substring(0, ++j); // opdracht
      s = s.Substring(j); // waarde
      if (k == "1K")
      { // opdracht 1K, waarde huiscode....
        cmbCode.Text = s;
        TransmitText("K2\n");
      }
      else if (k == "2K")
      {
        yyyy = int.Parse(s);
        TransmitText("K3\n");
      }
      else if (k == "3K")
      {
        MM = int.Parse(s);
        TransmitText("K4\n");
      }
      else if (k == "4K")
      {
        dd = int.Parse(s);
        TransmitText("K5\n");
      }
      else if (k == "5K")
      {
        hh = int.Parse(s);
        TransmitText("K6\n");
      }
      else if (k == "6K")
      {
        mm = int.Parse(s);
        TransmitText("K7\n");
      }
      else if (k == "7K")
      {
        ss = int.Parse(s);
        DateTime dt = new DateTime(yyyy, MM, dd, hh, mm, ss);
        rtc = DateTime.Now - dt;  // verschil in tijd bewaren
        lblRTC.Text = dt.ToString("dd-MM-yyyy HH:mm");
        TransmitText("K8\n");
      }
      else if (k == "8K")
      {
        for (int i = 1; i <= 16; i++)
        {  // vink alle lampen aan/uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chb" + i.ToString()];
          checkbox.Checked = (s.Substring(i - 1, 1).ToString() == "1");
        }
        TransmitText("K9\n");
      }
      else if (k == "9K")
      {
        for (int i = 1; i <= 16; i++)
        {  // vink alle DIM aan/uit
          CheckBox checkbox = (CheckBox)this.grbLicht.Controls["chbD" + i.ToString()];
          checkbox.Checked = (s.Substring(i - 1, 1).ToString() == "1");
        }
        TransmitText("K10\n");
      }
      else if (k == "10K")
      {
        lblOp.Text = s + ":";
        TransmitText("K11\n");
      }
      else if (k == "11K")
      {
        lblOp.Text += s;
        TransmitText("K12\n");
      }
      else if (k == "12K")
      {
        lblOnder.Text = s + ":";
        TransmitText("K13\n");
      }
      else if (k == "13K")
      {
        lblOnder.Text += s;
        TransmitText("K14\n");
      }
      else if (k == "14K")
      {
        cmbOffset.Text = s;
        TransmitText("K15\n");
      }
      else if (k == "15K")
      {
        chbOchtend.Checked = s == "1";
        TransmitText("K16\n");
      }
      else if (k == "16K")
      {
        chbAvond.Checked = s == "1";
        TransmitText("K17\n");
      }
      else if (k == "17K")
      {
        string l = s.Length == 1 ? "0" : "";
        cmbAan.Text = l + s + ":00";
        TransmitText("K18\n");
      }
      else if (k == "18K")
      {
        string l = s.Length == 1 ? "0" : "";
        try
        {
          cmbAan.Text = cmbAan.Text.Substring(0, 3) + l + s;
        }
        catch (Exception)
        {
          cmbAan.Text = "07:00";
        }
        TransmitText("K19\n");
      }
      else if (k == "19K")
      {
        string l = s.Length == 1 ? "0" : "";
        cmbUit.Text = l + s + ":00";
        TransmitText("K20\n");
      }
      else if (k == "20K")
      {
        string l = s.Length == 1 ? "0" : "";
        try
        {
          cmbUit.Text = cmbUit.Text.Substring(0, 3) + l + s;
        }
        catch (Exception)
        {
          cmbUit.Text = "22:00";
        }
      }
    }

    private void TransmitText(string verzenden)
    { // opdrachten verzenden
      if (serialPort1.IsOpen && verzenden.Length > 0)
      {
        try
        {
          serialPort1.Write(verzenden);
          System.Threading.Thread.Sleep(100);
        }
        catch (Exception)
        {
          MessageBox.Show("Helaas: verzenden mislukt\n\nWas het een Update?\nDan Arduino eerst resetten");
        }
      }
    }


  }
}
